!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   DEMのための相互関数法によるピーク決定プログラム non-BG_removal
!                DEM_cc.f90   by K.Suzuki, Niigata University
!                  18 August, 2023
!  SUBROUTINEのヘッダーの変数群に実験の数値をセットすること●●●●●●●
!     z, y方向の共用プログラムを作成
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  program DEM_cc
    
    implicit none
    
    real*8 :: ra, twth,r1,r2,c
    real*8 ::r(0:1000), tth(0:1000)
    
    integer :: i1,i2,start,cycle,qt,j,dir
    
    print *,"DEM_cor の下にtargetP1, targetP2の各フォルダがあり, "
    print *,"前処理で作成された x_i.txt群が用意されてますか?"
    print *,"DEM_cc.f90と同じフォルダにcc_fit.batがあり，実行型になっていますか?"
    print *,"OK (1)  or STOP (0)"
    read(5,*) start
    if (start .ne. 1) then
      stop
    end if
    
    print *,"処理する方向を入力してください．z=0, y=1"
    print *,"z=方向は垂直(検出器x方向) y=方向は垂直(検出器y方向)"
    read(5,*) dir
    
    print *,"処理する開始番号を入力してください"
    read(5,*) i1
    print *,"処理を停止する番号を入力してください"
    read(5,*) i2
    
    print *,"gnuplotの画面出力を見たい: 1  or  pdf出力のみ:0"
    read(5,*) qt     
    
    print *,"BG除去したい: 0  or  しない:1"
    read(5,*) j
    qt=qt+j*10            !qt>9 then 除去しない．
    
    open(unit=16,file="DEM_results.txt", status="unknown")
    write(16,*) i2-i1+1         !データ数
    
    do cycle=i1,i2
    
      call wave(cycle,dir)   ! 周積分の波形w1,w2をtargetP1, targetP2に出力
      
      call sokan(cycle,twth,ra,r1,r2,qt,c)
      r(cycle) = ra          !DEMによる回折半径
      tth(cycle) = twth      !DEMによる回折角(deg)
      
      write(16,*) cycle,tth(cycle),r(cycle),c,r1,r2
    end do  ! cycle
    
    close(16)
 612 format(i5,F10.6, F10.6, F6.4,F10.6,F10.6)
end program DEM_cc

!====================== SUBROUTINE ====================================
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   周積分波形 w1, w2 を作成する
!           SUBROUTINE   wave(i_roi)
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
subroutine wave(i_roi,dir)

  implicit none
  
  real*8 :: p1(0:500,0:200),p2(0:500,0:200)
  real*8 :: L0,L,pi=3.1415926,x0,y0,x02,y02,dr,dp
  real*8 :: r1, r2, phi,r1s,r1e,br,sv,bp
  real*8 :: rx1(0:200),rx2(0:200),w1(0:200),w2(0:200)
  
  integer :: p1c(0:500,0:200),p2c(0:500,0:200),dir
  
  integer :: i, j, k, m, n,i_roi, nod=150 !半径r1, r2のデータ数
  character fname*50,c_roi*5
  
  
  ! data table   ●●●●●●●●●● 実験値を入れる　78, 206, 243 行
  m=200; n=190       !copy_paste.ijmで取り出した画像寸法
  L0=510.d0; L=500.d0  ! これが最適値
  x0=105.368; y0=113.254; x02=119.902; y02=113.810 !ビームセンター座標
  r1s=82.d0; r1e=86.d0  ! SUS316の311回折環の回折半径 r1 波形w1の範囲
  bp=5.d0 !phi = +/- bp    +/-10はPILATUS, CdTe-pixelでは+/-5 deg
  
  !print *,"roiの番号を入れてください．"
  !read(5,*) i_roi

  !  P1, P2の読み込み．前処理のこと
  write(c_roi,'(I5)') i_roi !数字i_roiを文字に変換してc_roiに保存
  c_roi = adjustl(c_roi)  ! c_roiの文字の左詰め
  fname="targetP1/x_" // trim(c_roi) // ".txt" ! trimでc_roiの右空白を取る
    print *,fname
  open(file=fname,unit=11,status="old")

  do j=0,n
      read(11,*) (p1(i,j), i=0,m,1)
  end do
  close(11)
  
  fname="targetP2/x_" // trim(c_roi) // ".txt"
  print *,fname
  open(file=fname,unit=11,status="old")

  do j=0,n
      read(11,*) (p2(i,j), i=0,m,1)
  end do
  close(11)
  
  !'''''''''''''''  波形 W1(r1), W2(r2)の計算  ''''''''''''''''''''
  
  dr=(r1e-r1s)/float(nod)
  
  do j=0,n
   do i=0,m
    p1c(i,j)=0; p2c(i,j)=0 !積分領域表示配列p1c,p2cの初期化
   end do
  end do
  
  ! r1,W1の計算
  do i=0,nod
    r1=r1s+dr*float(i)
    
    sv=0.d0
    dp=2.d0*bp/400.d0
    do j=0,400
      phi=-bp + dp*float(j)
      call int_p1(dir,p1,r1,phi,br,p1c)
      sv=sv+br
    end do
    
    rx1(i)=r1  !r1のデータ
    w1(i)=sv   !W1のデータ
  end do

  ! r2,W2の計算
  do i=0,nod
    r1=r1s+dr*float(i)
    r2=r1*(1.d0+L/L0)
    sv=0.0
    dp=2.d0*bp/400.d0
    do j=0,400
      phi=-bp + dp*float(j)
      call int_p2(dir,p2,r2,phi,br,p2c)
      sv=sv+br
    end do
    
    rx2(i)=r2  !r2のデータ
    w2(i)=sv   !W2のデータ
!    print *,rx2(i),w2(i)
  end do
  
  !   積分利用域の出力
  
  do j=0,n
    do i=0,m
        p1c(i,j) = p1c(i,j)*80 + int(p1(i,j))
        p2c(i,j) = p2c(i,j)*80 + int(p2(i,j))
    end do
  end do
  
    fname="targetP1/x1_" // trim(c_roi) // ".txt"
    open(file=fname,unit=11,status="unknown")
     do j=0,n
      write(11,*) (p1c(i,j), i=0,m)
     end do
    close(11)
    
    fname="targetP2/x2_" // trim(c_roi) // ".txt"
    open(file=fname,unit=11,status="unknown")
     do j=0,n
      write(11,*) (p2c(i,j), i=0,m)
     end do
    close(11)
    
  !~~~~~~~~~~~~~~~~~~~~~ P1, P2波形の出力 ~~~~~~~~~~~~~
  fname= "targetP1/w1_" // trim(c_roi) // ".txt"
  open(file=fname,unit=12,status="unknown")
     do i=0,nod
      write(12,*) rx1(i),w1(i)
     end do
  close(12)

  
  fname= "targetP2/w2_" // trim(c_roi) // ".txt"
  open(file=fname,unit=12,status="unknown")
     do i=0,nod
      write(12,*) rx2(i),w2(i)
     end do
  close(12)
    
end subroutine wave
  
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   subroutine int_p1  I(r1,phi):P1のカウントを求める
!  SUBROUTINEのヘッダーの変数群に実験の数値をセットすること●●●●●
subroutine int_p1(dir,p,r,phi,cnt,c)
  
    implicit none
  
    real*8 :: p(0:500,0:200), r, x,y, alpha, beta,cnt,cnt1,cnt2
    real*8 :: L0,L,P1h,pi, phi,x0,y0,x02,y02,fi,fth,sr
    real*8 :: i11,i12,i21,i22

    integer:: i, j, m, n,c(0:500,0:200),dir
    
    !2023/06/22     変数群に実験の数値をただ新セットすること●●●●●
    m=200; n=190
    L0=510.d0; L=500.d0 ; pi=3.1415926  ! これが最適値
    x0=105.368; y0=113.254; x02=119.902; y02=113.810 !ビームセンター座標
    P1h=83.d0   ! 検出器高さ
    sr=0.2d0 ! mm/pixel
    fi=phi*pi/180.d0 
    
    if (dir == 0) then
      x=x0 + r*sin(fi)/sr
      y=y0 - (r*cos(fi) - P1h)/sr  
    else
      x=x0 + (r*cos(fi)-P1h)/sr
      y=y0+r*sin(fi)/sr
    end if
    
    i=int(x); j=int(y)
!      print *,i,j    !積分範囲の確認用 i,jが配列範囲を超えないこと
    alpha=x-float(i); beta=y-float(j)
  i11=p(i,j)
  i12=p(i+1,j)
  i21=p(i,j+1)
  i22=p(i+1,j+1)
  
  cnt1 = (1.-alpha)*(1.-beta)*i11 + alpha*(1.-beta)*i12
  cnt2 = (1.-alpha)*beta*i21 + alpha*beta*i22
  cnt = cnt1+cnt2  !intensity of (x,y)
    c(i,j)=1
end subroutine int_p1

!   ~~~~~~~~~~~~~~~~~~~~~~~~
!   subroutine int_p2  I(r2,phi):P2のカウントを求める
!  SUBROUTINEのヘッダーの変数群に実験の数値をセットすること●●●●●
subroutine int_p2(dir,p,r,phi,cnt,c)
  
    implicit none
  
    real*8 :: p(0:500,0:200), r, x,y, alpha, beta,cnt,cnt1,cnt2
    real*8 :: L0,L,P2h,P2hy,pi, phi,x0,y0,x02,y02,fi,fth,sr
    real*8 :: i11,i12,i21,i22
    integer:: i, j, m, n,c(0:500,0:200),dir
    
    !2023/06/22
    m=200; n=190
    L0=510.d0; L=500.d0; pi=3.1415926d0  ! これが最適値
    x0=105.368; y0=113.254; x02=119.902; y02=113.810 !ビームセンター座標
    P2h=166.d0; P2hy=167.d0  !P2の検出器の高さ 
          !y方向の検出器の位置が合わなかったので，P2hy=167.0で定義した．
    sr=0.2d0 ! mm/pixel
    fi=phi*pi/180.d0 
    
    if (dir == 0) then
      x=x02 + r*sin(fi)/sr
      y=y02 - (r*cos(fi) - P2h)/sr 
    else
      x=x02 + (r*cos(fi) - P2hy)/sr
      y=y02 + r*sin(fi)/sr
    end if
    
    i=int(x); j=int(y)
  !    print *,i,j
    alpha=x-float(i); beta=y-float(j)
  i11=p(i,j)
  i12=p(i+1,j)
  i21=p(i,j+1)
  i22=p(i+1,j+1)
  
  cnt1 = (1.-alpha)*(1.-beta)*i11 + alpha*(1.-beta)*i12
  cnt2 = (1.-alpha)*beta*i21 + alpha*beta*i22
  cnt = cnt1+cnt2  !intensity of (x,y)
    c(i,j)=1
end subroutine int_p2

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!      sokan(i_1,ttho,ro,r1o,r2o)
!     改変された相関係数による波形の一致から回折角を評価
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

subroutine sokan(i_1,ttho,ro,r1o,r2o,qt,c_max)
  
    implicit none
    
    real*8 :: w1(0:200),r1(0:200),w2(0:200),r2(0:200)
    real*8 :: x(0:200),y(0:200),c(0:200)
    real*8 :: xm,ym,ym0,sdx,sdy,sxy, w1max, w2max, sm, c_max
    real*8 :: a, b, c1,c11,c12,c2   !直線・放物近似
    real*8 :: aa,bb,cc
    real*8 :: sum1,sum2,sum3,sum4,sum5,sum6,sum7
    real*8 :: ma(1:100,1:100),mb(1:100), fit,r1o,r2o,ttho,ro
    real*8 :: L=500.d0           ! DEM実験のLの値を入力する ●●●●●●●●●●●●●●●●
    
    integer :: i,j,k,iw1_max,iw2_max,k_max,i_1,kk
    integer :: in=20,ib=41,m,np=3,nod=150 
               !波形幅ib,半幅in,m,放物点数np,データ数nod ●●●●● ibは波形幅にする
               !基準波形の母に合わせて半幅inを決め，ib=2*in+1とする．
    integer :: qt  !gnuplotの画面出力 qt=1
    
    character fname*30, c_1*5,cmd*150,fitting*25
    
    !print *,"w1データファイルの番号を入れてください．"
    !read(5,*) i_1
    write(c_1,'(I5)') i_1 !数字i_1を文字に変換してc_1に保存
    c_1 = adjustl(c_1)  ! c_roiの文字の左詰め
    fname="targetP1/w1_" // trim(c_1) // ".txt" ! trimでc_roiの右空白を取る
    print *,fname
    open(file=fname,unit=11,status="old")
  
    do i=0,nod
      read(11,*) r1(i),w1(i)
    end do
    close(11)
    
    fname="targetP2/w2_" // trim(c_1) // ".txt" ! trimでc_roiの右空白を取る
    print *,fname
    open(file=fname,unit=11,status="old")
  
    do i=0,nod
      read(11,*) r2(i),w2(i)
!      print *,w2(i)
    end do
    close(11)
    
    
    ! removing background
    if(qt < 3 ) then
    ! for w1
      c1=0.d0; c11=0.d0; c12=0.d0; c2=0.d0
    
      do i=0,2
        c1=c1+w1(i)*r1(i); c11=c11+r1(i)*r1(i); c12=c12+r1(i); c2=c2+w1(i)
      end do
      do i=nod-2,nod               !  i=nod-2,nod ●●●●●●●●●●●●●●
        c1=c1+w1(i)*r1(i); c11=c11+r1(i)*r1(i); c12=c12+r1(i); c2=c2+w1(i)
      end do
      a=(c1-1./6.*c12*c2)/(c11-1./6.*c12*c12)
      b=(c2-c12*a)/6.d0
    
      do i=0,nod
        w1(i)=w1(i)-a*r1(i)-b
    !  print *,r1(i),w1(i)
      end do
    
    ! for w2
      c1=0.d0; c11=0.d0; c12=0.d0; c2=0.d0
    
      do i=0,2
        c1=c1+w2(i)*r2(i); c11=c11+r2(i)*r2(i); c12=c12+r2(i); c2=c2+w2(i)
      end do
      do i=nod-2,nod  !  i=nod-2,nod ●●●●●●●●●●●●●●●●●●●●●●●●●●●●●●●●
        c1=c1+w2(i)*r2(i); c11=c11+r2(i)*r2(i); c12=c12+r2(i); c2=c2+w2(i)
      end do
      a=(c1-1./6.*c12*c2)/(c11-1./6.*c12*c12)
      b=(c2-c12*a)/6.d0
    
      do i=0,nod
        w2(i)=w2(i)-a*r2(i)-b
!      print *,r2(i),w2(i)
      end do    
    end if
    
    ! searching maxima of w1 and w2
    w1max=w1(1); iw1_max=1
    w2max=w2(1); iw2_max=1
    do i=1,nod
      if (w1(i) > w1max) then
        w1max=w1(i); iw1_max=i
      end if
      if (w2(i) > w2max) then
        w2max=w2(i); iw2_max=i
      end if
    end do
    
    !normalizing
    do i=0,nod
      w1(i)=w1(i)/w1max
      w2(i)=w2(i)/w2max
!     print *,w2(i)
    end do
    
    ! calculating correlation coefficients
    do i=0,ib-1
        j=iw1_max-in+i
      x(i)=w1(j)   !w1の基準波形x
!      print *,i,x(i) 
    end do
    !平均
    sm=0.d0
    do i=0,ib-1
        sm=sm+x(i)
    end do
    xm=sm/float(ib)
    
    sm=0.d0
    do i=iw2_max-in,iw2_max+in
        sm=sm+w2(i)
    end do
    ym0=sm/float(ib)
      
    !偏差
    sm=0.d0
    do i=0,ib-1
        sm=sm+(x(i)-xm)**2
    end do
    sdx=sqrt(sm)  !偏差x
    
    sm=0.d0
    do i=iw2_max-in,iw2_max+in
        sm=sm+(w2(i)-ym0)**2
    end do
    sdy=sqrt(sm)  !偏差y
    
    
    !相互相関cの初期化(繰り返しのとき必要になる)
    do i=1,nod
        c(i)=0.d0
    end do
    
    !   kで相互相関cのループ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    do k=1,nod-ib
      do i=0,ib-1
        j=k+i
        y(i)=w2(j)   !w2の検出対象の波形y
      end do
      
      !平均
      sm=0.d0
      do i=0,ib-1
        sm=sm+x(i)
      end do
      xm=sm/float(ib)
      
      sm=0.d0
      do i=0,ib-1
        sm=sm+y(i)
      end do
      ym=sm/float(ib)
      
      !共分散
      sm=0.d0
      do i=0,ib-1
        sm=sm+(x(i)-xm)*(y(i)-ym)
      end do
      sxy=sm
      
      kk=k+in
      c(kk)=sxy/sdx/sdy
!      print *,c(k)
      
    end do  !  k         ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    !  相互相関の曲線を c_i.txtで保存
    fname="c_" // trim(c_1) // ".txt" ! trimでc_1の右空白を取る
    print *,fname
    open(file=fname,unit=12,status="unknown")
      write(12,*) nod
    do k=1,nod
        write(12,*) k,c(k),r1(k),w1(k),r2(k),w2(k)
      end do
    close(12)
    
    k_max=in+1; c_max=c(in)
    do k=in+1,nod-ib
      if(c(k) > c_max) then
        c_max=c(k); k_max=k
      end if
    end do
    
    !放物近似で頂点の値を決める
  sum1=0.d0;sum2=0.d0;sum3=0.d0;sum4=0.d0
  sum5=0.d0;sum6=0.d0;sum7=0.d0
  
  j=2*np+1
  do i=k_max-np,k_max+np
    sum1=sum1+float(i)**4
    sum2=sum2+float(i)**3
    sum3=sum3+float(i)**2
    sum4=sum4+float(i)
    sum5=sum5+c(i)*float(i)**2
    sum6=sum6+c(i)*float(i)
    sum7=sum7+c(i)
  end do
    ma(1,1)=sum1;   ma(1,2)=sum2;   ma(1,3)=sum3
    ma(2,1)=ma(1,2);ma(2,2)=ma(1,3);ma(2,3)=sum4
    ma(3,1)=ma(1,3);ma(3,2)=ma(2,3);ma(3,3)=float(j)
  
    mb(1)=sum5
    mb(2)=sum6
    mb(3)=sum7
  
  m=3   !未知数3,放物線
  call GAUSS(ma,mb,m)
  aa=mb(1)
  bb=mb(2)
  cc=mb(3)
  fit=-bb/2./aa
  
!print *,fit
  
  a = (r2(100)-r2(0))/100.
  b = r2(0)
  r1o=r1(iw1_max)    ! 波形w1のトップを基準
  r2o=a*fit+b        ! 波形w2の相互相関の最大値
  ro=r2o-r1o         ! DEMの回折半径
  ttho=atan(ro/L)*180.d0/3.1415926d0
  
  !write(6,*) r1o,r2o,a,b,fit
  !write(6,*) ro,ttho   !二重露光法による回折半径と回折角度
  
  
  !関係曲線のgnuplotの出力
  open(unit=12,file="ccc.plt",status="unknown")
  !print *,"ccc.plt"
  
  write(12,*) "set term pdf "
  fitting="ccc_"//trim(adjustl(c_1))//".pdf"
  cmd="set output '"//trim(fitting)//"'"
  write(12,*) trim(cmd)
  
  cmd="plot '"//trim(adjustl(fname))//"' using 1:2 with linespoints,"//"'" & 
  //trim(adjustl(fname))//"' using 1:4 with linespoints,"//"'"// &
  trim(adjustl(fname))//"' using 1:6 with linespoints"
  write(12,*) trim(cmd)
  write(12,*) "set term 'qt'"
  
  if (mod(qt,2) .eq. 1) then 
    cmd="plot '"//trim(adjustl(fname))//"' using 1:2 with linespoints,"//"'" & 
    //trim(adjustl(fname))//"' using 1:4 with linespoints,"//"'"// &
    trim(adjustl(fname))//"' using 1:6 with linespoints"
    write(12,*) trim(cmd)
    cmd="pause -1 '改行キーを押して戻る．'"
    write(12,*) trim(cmd); print *, cmd
  end if
    
  close(12)
  
    call SYSTEM('./cc_fit.bat')
    
end subroutine sokan    
!___________________________________________________
!  Solving simultaneous equations
!  using Gauss method
!         CALL GAUSS by Yui Igarashi 
!___________________________________________________

subroutine GAUSS(A,B,N)

      implicit none
      real*8:: A(1:100,1:100),B(1:100),P,Q
      integer:: I,J,K,L,N

!                     FRONT PROCESS
!                     SETTING L-LINE
      do L=1,N
        P=A(L,L)
        do I=L,N
          A(L,I)=A(L,I)/P
        end do
        B(L)=B(L)/P
!                     SUBSTITUTION
        do  I=L+1,N
          Q=A(I,L)
          do J=L,N
            A(I,J)=A(I,J)-Q*A(L,J)
          end do
          B(I)=B(I)-Q*B(L)
        end do
      end do
!           POST PROCESS

!                     SETTING L-LINE
      do L=N,1,-1
        P=A(L,L)
        do I=N,L,-1
          A(L,I)=A(L,I)/P
        end do
        B(L)=B(L)/P
!                     SUBSTITUTION
        do I=L-1,1,-1
          Q=A(I,L)
          do J=N,I,-1
            A(I,J)=A(I,J)-Q*A(L,J)
          end do
          B(I)=B(I)-Q*B(L)
        end do
      end do

end subroutine GAUSS 
