!*******************************************************************
!  CaTe pixel detectorのimgから画像x_i.txtを作成する．異常セルの補正もします．
!  作成されたx_i.txtからFijiのマクロmake_stack.ijmで stack_x.tifを作成して
!  画像チェックできます．
!
!            make_image.f90
!
!            by K.Suzuki, Niigata University, 31st July, 2023
!*******************************************************************

program make_image
  implicit none

  character filename*128
  character  bun*128
  
  integer i,j,k,l,m,n,loop,i1,i2,ie,ii,jj,maisu
  integer*4 :: x(0:94,0:399)    !32bit data
  integer*4 :: y(0:200,0:190)   !32bit data
    integer*4 :: z(0:200,0:190)   !32bit data
  
  print *,"このプログラムを処理するフォルダーから起動してください．"
  print *,"これからCdTepixel検出器のimg_i.txtフアイルを処理します．"
  print *,"応力測定のブロックごとに1番から連番で出力します．"
  print *,"測定ブロックごとに処理してください．"
  print *,"開始する番号(i1)，停止(i2)する番号を入力してください． 例  0,150 "
  read(5,*) i1,i2
  print *,"重ねる画像の枚数 maisu"
  read(5,*) maisu
  
  ie=(i2-i1+1)/maisu
  if (mod(i2,maisu) .ne. 0) then
    print *, "i2/maisu is not 0, error"; stop
  end if
  
  k=i1  !img  No.k
  
  do loop=1,ie   
    
    !初期化
    do j=0,190
      do i=0,200
        z(i,j)=0
      end do
    end do
    
    do l=1,maisu
      
      write(filename,*) k
      filename="img_"//trim(adjustl(filename))//".txt"
      
      open(unit=11,file=filename,status="old")
        do i=1,13
        read(11,*) bun
        end do
        do j=0,399
          do i=0,94
          read(11,*) ii,jj,x(i,j)
          end do
        end do
      close(11)
      
      call compose(x,y) ! 読み取った配列xから画像配列に変換y
      
      !補正
      
      do j=0,190
        do i=0,200
        z(i,j) = z(i,j) + y(i,j)
        end do
      end do
      
      k = k + 1
    end do  !l:枚数
    
    call improve(z)  ! 異常セルの補正
          
    !          ------------ 画像ファイル x_i.txt の出力 -----------
    write(filename,*) loop
    filename="x_"//trim(adjustl(filename))//".txt"
    
    open(unit=12,file=filename,status="unknown")
    
    do j=0,190
      write(12,*) (z(i,j),i=0,200)
    end do
    
    close(12)
    
  end do   ! loop

end program make_image

!****************************************************
subroutine compose(x,y)
  implicit none
  
  integer*4 :: x(0:94,0:399)    !32bit data
  integer*4 :: y(0:200,0:190)   !32bit data
  integer i,j,k,l,m,n
  integer*4 :: I1,I2,I3,I4
! ----------------------- compose data file
! segment c ->1, B -> 2

      n=0

!m=0-99:C, m=100:non, m=101-200: B
      do i=0,94,1
      m=0
      do j=299,200,-1
        y(m,n)=x(i,j)
        m=m+1
      end do
      m=m+1  ! 100 boundary pixel 
      do j=199,100,-1
        y(m,n)=x(i,j)
        m=m+1
      end do
      n=n+1
      end do

      n=n+1  ! 100 boundary pixel  

      do i=94,0,-1
        m=0
        do j=300,399,1
          y(m,n)=x(i,j)
          m=m+1
        end do
        m=m+1  ! 100 boundary pixel 
        do j=0,99,1
          y(m,n)=x(i,j)
        m=m+1
        end do
        n=n+1
      end do

! calculation counts at the boundaries
      do j=0,93
        y(99,j)=int(y(99,j)*2/3)
        y(101,j)=int(y(101,j)*2/3)
        y(100,j)=int((y(99,j)+y(101,j))/2)
      end do
      do j=97,190
        y(99,j)=int(y(99,j)*2/3)
        y(101,j)=int(y(101,j)*2/3)
        y(100,j)=int((y(99,j)+y(101,j))/2)
      end do

      do i=0,98
        y(i,94)=int(y(i,94)*2/3)
        y(i,96)=int(y(i,96)*2/3)
        y(i,95)=int((y(i,94)+y(i,96))/2)
      end do
      do i=102,200
        y(i,94)=int(y(i,94)*2/3)
        y(i,96)=int(y(i,96)*2/3)
        y(i,95)=int((y(i,94)+y(i,96))/2)
      end do

      I1=y(99,94)
      I2=y(101,94)
      I3=y(99,96)
      I4=y(101,96)

      y(99,94)=int(I1*4/9)
      y(100,94)=int((I1+I2)*2/9)
      y(101,94)=int(I2*4/9)
      
      y(99,95)=int((I1+I3)*4/9)
      y(100,95)=int((I1+I2+I3+I4)/9)
      y(101,95)=int((I2+I4)*4/9)

      y(99,96)=int(I3*4/9)
      y(100,96)=int((I3+I4)*2/9)
      y(101,96)=int(I4*4/9)

end subroutine compose




! ***********************************
!  ピクセル異常の修正
subroutine improve(z)
  implicit none
  
    integer*4 :: z(0:200,0:190),t(0:200,0:190),x(1:9)   !32bit data
    integer*4 :: p
    integer :: i,j,k,l,m,n
    
    !境界は対象外
    do j=1,189
      do i=1,199
      !補正値tの計算  異常画素に中間値を入れる
        if ( (z(i,j) == 0) .or. (ABS(z(i,j)) > 1d5) ) then !異常セル
         m=1
         do l=j-1,j+1
          do k=i-1,i+1
            x(m)=z(k,l)
            m=m+1
          end do
         end do
         
         !sorting x
         do k=9,2,-1
          do l=2,k
            if(x(l-1) >x(l)) then
              p=x(l); x(l)=x(l-1); x(l-1)=p
            end if
          end do
          end do
          t(i,j)=x(5)  !中間値の入力
        else                              !正常セル
          t(i,j)=z(i,j)
        end if
      end do !i
    end do !j
    
    !補正画像に入れ替え
    do j=1,189
      do i=1,199
        z(i,j)=t(i,j)
      end do
    end do

end subroutine improve